* {
    box-sizing: border-box;
}

body {
    all: unset;
    min-height: 100vh;
    display: flex;
    background-color: #0b0f19;
    font-family: "Source Sans Pro", "ui-sans-serif", "system-ui", sans-serif;
    text-align: center;
}

nav {
    width: 15%;
}

main {
    width: 85%;
}

iframe {
    all: unset;
    width: 100%;
    height: 100%;
}

nav ul {
    all: unset;
    display: flex;
    flex-direction: column;
    padding: 16px 8px;
}

nav li {
    all: unset;
    display: flex;
    margin-bottom: 10px;
}

nav li.active, nav li a:hover {
    background-color: #f97316;
    border-radius: 8px;
}

nav li a {
    all: unset;
    color: white;
    padding: 10px 10px;
    width: 100%;
}