"""
Utility functions for dashboard pages.
"""

CSS = """
h2 {
    text-align: center;
    display: block;
}

#code_block {
    height: 60vh;
    overflow-y: hidden;
    margin-bottom: 30px;
}

div.toast-body.error {
    top: 50% !important;
    left: 50% !important;
    transform: translate(-50%, -50%) !important;
    position: fixed !important;
    z-index: 9999 !important;
}

div.toast-body.info {
    top: 50% !important;
    left: 50% !important;
    transform: translate(-50%, -50%) !important;
    position: fixed !important;
    z-index: 9999 !important;
}
"""
